@@ -4,45 +4,18 @@
 import argparse
 import glob
 import io
-import locale
 import os
 import re
-import shutil
 import sys
-import uuid
 
 import parso
 import tbtrim
+from bpc_utils import BOOLEAN_STATES, CPU_CNT, LOCALE_ENCODING, UUID4Generator, archive_files, detect_files, mp
 
 __all__ = ['walrus', 'convert']
 
-# multiprocessing may not be supported
-try:        # try first
-    import multiprocessing
-except ImportError:  # pragma: no cover
-    multiprocessing = None
-else:       # CPU number if multiprocessing supported
-    if os.name == 'posix' and 'SC_NPROCESSORS_CONF' in os.sysconf_names:  # pragma: no cover
-        CPU_CNT = os.sysconf('SC_NPROCESSORS_CONF')
-    elif hasattr(os, 'sched_getaffinity'):  # pragma: no cover
-        CPU_CNT = len(os.sched_getaffinity(0))  # pylint: disable=E1101
-    else:  # pragma: no cover
-        CPU_CNT = os.cpu_count() or 1
-finally:    # alias and aftermath
-    mp = multiprocessing
-    del multiprocessing
-
 # version string
-__version__ = '0.1.2'
-
-# from configparser
-BOOLEAN_STATES = {'1': True, '0': False,
-                  'yes': True, 'no': False,
-                  'true': True, 'false': False,
-                  'on': True, 'off': False}
-
-# environs
-LOCALE_ENCODING = locale.getpreferredencoding(False)
+__version__ = '0.1.4'
 
 # macros
 grammar_regex = re.compile(r"grammar(\d)(\d)\.txt")
@@ -51,6 +24,9 @@ WALRUS_VERSION = sorted(filter(lambda version: version >= '3.8',  # when Python
                                    glob.glob(os.path.join(parso.__path__[0], 'python', 'grammar??.txt')))))
 del grammar_regex
 
+# will be set in every call to `convert()`
+uuid_gen = None
+
 
 class ConvertError(SyntaxError):
     """Parso syntax error."""
@@ -79,43 +55,6 @@ def predicate(filename):  # pragma: no cover
 
 tbtrim.set_trim_rule(predicate, strict=True, target=(ConvertError, ContextError))
 
-###############################################################################
-# UUID 4 generator wrapper
-
-
-class UUID4Generator:
-    """UUID 4 generator wrapper to prevent UUID collisions."""
-
-    def __init__(self, dash=True):
-        """Constructor of UUID 4 generator wrapper.
-
-        Args:
-         - `dash` -- `bool`, whether the generated UUID string has dashes or not
-
-        """
-        self.used_uuids = set()
-        self.dash = dash
-
-    def gen(self):
-        """Generate a new UUID 4 string that is guaranteed not to collide with used UUIDs.
-
-        Returns:
-         - `str` -- a new UUID 4 string
-
-        """
-        while True:
-            nuid = uuid.uuid4()
-            nuid = str(nuid) if self.dash else nuid.hex
-            if nuid not in self.used_uuids:
-                break
-        self.used_uuids.add(nuid)
-        return nuid
-
-
-# Initialise a UUID4Generator for filename UUIDs.
-# In the conversion process, this will be replaced by a new UUID4Generator for identifier UUIDs.
-uuid_gen = UUID4Generator(dash=True)
-
 ###############################################################################
 # Main convertion implementation
 
@@ -172,13 +111,12 @@ CLS_FUNC_TEMPLATE = '''\
 '''.splitlines()  # `str.splitlines` will remove trailing newline
 
 
-def parse(string, source, error_recovery=False):
+def parse(string, source):
     """Parse source string.
 
     Args:
      - `string` -- `str`, context to be converted
      - `source` -- `str`, source of the context
-     - `error_recovery` -- `bool`, see `parso.Grammar.parse`
 
     Envs:
      - `WALRUS_VERSION` -- convert against Python version (same as `--python` option in CLI)
@@ -187,16 +125,18 @@ def parse(string, source, error_recovery=False):
      - `parso.python.tree.Module` -- parso AST
 
     Raises:
-     - `ConvertError` -- when `parso.ParserSyntaxError` raised
+     - `ConvertError` -- when source code contains syntax errors
 
     """
-    try:
-        return parso.parse(string, error_recovery=error_recovery,
-                           version=os.getenv('WALRUS_VERSION', WALRUS_VERSION[-1]))
-    except parso.ParserSyntaxError as error:
-        message = '%s: <%s: %r> from %s' % (error.message, error.error_leaf.token_type,
-                                            error.error_leaf.value, source)
-        raise ConvertError(message).with_traceback(error.__traceback__) from None
+    grammar = parso.load_grammar(version=os.getenv('WALRUS_VERSION', WALRUS_VERSION[-1]))
+    module = grammar.parse(string, error_recovery=True)
+    errors = grammar.iter_errors(module)
+
+    if errors:
+        error_messages = '\n'.join('[L%dC%d] %s' % (*error.start_pos, error.message) for error in errors)
+        raise ConvertError('source file %r contains following syntax errors:\n' % source + error_messages)
+
+    return module
 
 
 class Context:
@@ -512,19 +452,10 @@ class Context:
          - `node` -- `parso.python.tree.Function`, function node
 
         """
-        # 'def' NAME '(' PARAM ')' ':' SUITE
-        func_def, func_name, func_param, func_op, func_suite = node.children
-
-        # <Keyword: def>
-        self._process(func_def)
-        # <Name: ...>
-        self._process(func_name)
-        # PythonNode(parameters, [...])
-        self._process(func_param)
-        # <Operator: :>
-        self._process(func_op)
-        # suite
-        self._process_suite_node(func_suite, func=True)
+        # 'def' NAME '(' PARAM ')' [ '->' NAME ] ':' SUITE
+        for child in node.children[:-1]:
+            self._process(child)
+        self._process_suite_node(node.children[-1], func=True)
 
     def _process_lambdef(self, node):
         """Process lambda definition (``lambdef``).
@@ -913,7 +844,7 @@ class Context:
         if pool[sort[0]] > pool[sort[1]]:
             return sort[0]
 
-        env = os.getenv('POSEUR_LINESEP', os.linesep)
+        env = os.getenv('WALRUS_LINESEP', os.linesep)
         env_name = env.upper()
         if env_name == 'CR':
             return '\r'
@@ -923,7 +854,7 @@ class Context:
             return '\n'
         if env in ['\r', '\r\n', '\n']:
             return env
-        raise EnvironError('invlid line separator %r' % env)
+        raise EnvironError('invalid line separator %r' % env)
 
     @staticmethod
     def is_walrus(node):
@@ -1359,73 +1290,35 @@ def get_parser():
                                      usage='walrus [options] <python source files and folders...>',
                                      description='Back-port compiler for Python 3.8 assignment expressions.')
     parser.add_argument('-V', '--version', action='version', version=__version__)
-    parser.add_argument('-q', '--quiet', action='store_true',
-                        help='run in quiet mode')
+    parser.add_argument('-q', '--quiet', action='store_true', help='run in quiet mode')
 
     archive_group = parser.add_argument_group(title='archive options',
                                               description="duplicate original files in case there's any issue")
     archive_group.add_argument('-na', '--no-archive', action='store_false', dest='archive',
                                help='do not archive original files')
     archive_group.add_argument('-p', '--archive-path', action='store', default=__archive__, metavar='PATH',
-                               help='path to archive original files (%s)' % __archive__)
+                               help='path to archive original files (%(default)s)')
 
     convert_group = parser.add_argument_group(title='convert options',
-                                              description='compatibility configuration for none-unicode files')
+                                              description='compatibility configuration for non-unicode files')
     convert_group.add_argument('-c', '--encoding', action='store', default=__walrus_encoding__, metavar='CODING',
-                               help='encoding to open source files (%s)' % __walrus_encoding__)
+                               help='encoding to open source files (%(default)s)')
     convert_group.add_argument('-v', '--python', action='store', metavar='VERSION',
                                default=__walrus_version__, choices=WALRUS_VERSION,
-                               help='convert against Python version (%s)' % __walrus_version__)
+                               help='convert against Python version (%(default)s)')
     convert_group.add_argument('-s', '--linesep', action='store', default=__walrus_linesep__, metavar='SEP',
-                               help='line separator to process source files (%r)' % __walrus_linesep__)
+                               help='line separator to process source files (%(default)r)')
     convert_group.add_argument('-nl', '--no-linting', action='store_false', dest='linting',
                                help='do not lint converted codes')
     convert_group.add_argument('-t', '--tabsize', action='store', default=__walrus_tabsize__, metavar='INDENT',
-                               help='indentation tab size (%s)' % __walrus_tabsize__, type=int)
+                               help='indentation tab size (%(default)s)', type=int)
 
     parser.add_argument('file', nargs='+', metavar='SOURCE', default=__cwd__,
-                        help='python source files and folders to be converted (%s)' % __cwd__)
+                        help='python source files and folders to be converted (%(default)s)')
 
     return parser
 
 
-def find(root):  # pragma: no cover
-    """Recursively find all files under root.
-
-    Args:
-     - `root` -- `os.PathLike`, root path to search
-
-    Returns:
-     - `Generator[str, None, None]` -- yield all files under the root path
-
-    """
-    file_list = list()
-    for entry in os.scandir(root):
-        if entry.is_dir():
-            file_list.extend(find(entry.path))
-        elif entry.is_file():
-            file_list.append(entry.path)
-        elif entry.is_symlink():  # exclude symbolic links
-            continue
-    yield from file_list
-
-
-def rename(path, root):
-    """Rename file for archiving.
-
-    Args:
-     - `path` -- `os.PathLike`, file to rename
-     - `root` -- `os.PathLike`, archive path
-
-    Returns:
-     - `str` -- the archiving path
-
-    """
-    stem, ext = os.path.splitext(path)
-    name = '%s-%s%s' % (stem, uuid_gen.gen(), ext)
-    return os.path.join(root, name)
-
-
 def main(argv=None):
     """Entry point for walrus.
 
@@ -1456,41 +1349,26 @@ def main(argv=None):
 
     linesep = args.linesep.upper()
     if linesep == 'CR':
-        os.environ['POSEUR_LINESEP'] = '\r'
+        os.environ['WALRUS_LINESEP'] = '\r'
     elif linesep == 'CRLF':
-        os.environ['POSEUR_LINESEP'] = '\r\n'
+        os.environ['WALRUS_LINESEP'] = '\r\n'
     elif linesep == 'LF':
-        os.environ['POSEUR_LINESEP'] = '\n'
+        os.environ['WALRUS_LINESEP'] = '\n'
     elif args.linesep in ['\r', '\r\n', '\n']:
-        os.environ['POSEUR_LINESEP'] = args.linesep
+        os.environ['WALRUS_LINESEP'] = args.linesep
     else:
         raise EnvironError('invalid line separator %r' % args.linesep)
 
-    # make archive directory
-    if args.archive:  # pragma: no cover
-        os.makedirs(ARCHIVE, exist_ok=True)
-
     # fetch file list
-    filelist = list()
-    for path in args.file:
-        if os.path.isfile(path):
-            if args.archive:  # pragma: no cover
-                dest = rename(path, root=ARCHIVE)
-                os.makedirs(os.path.dirname(dest), exist_ok=True)
-                shutil.copy(path, dest)
-            filelist.append(path)
-        if os.path.isdir(path):  # pragma: no cover
-            if args.archive:
-                shutil.copytree(path, rename(path, root=ARCHIVE))
-            filelist.extend(find(path))
-
-    # check if file is Python source code
-    ispy = lambda file: (os.path.isfile(file) and (os.path.splitext(file)[1] in ('.py', '.pyw')))
-    filelist = sorted(filter(ispy, filelist))
+    filelist = sorted(detect_files(args.file))
 
     # if no file supplied
     if not filelist:  # pragma: no cover
-        parser.error('argument PATH: no valid source file found')
+        parser.error('no valid source file found')
+
+    # make archive
+    if args.archive:
+        archive_files(filelist, ARCHIVE)
 
     # process files
     if mp is None or CPU_CNT <= 1:
